import FluentAdFlowAdsWidget

@objc(FluentAdFlowWidgetViewManager)
class FluentAdFlowWidgetViewManager: RCTViewManager {

  override func view() -> (FluentAdFlowWidget) {
    return FluentAdFlowWidget()
  }

  @objc override static func requiresMainQueueSetup() -> Bool {
    return false
  }
}

class FluentAdFlowWidget : FluentAdFlowWidgetView {
  required init?(apiKey: String, referer: String) {
    super.init(apiKey: apiKey, referer: referer)
  }

  required init(frame: CGRect){
    super.init(frame: frame)
    super.events.listenTo(eventName: "onAdFlowContainerClosed", action: onAdFlowContainerClosed)
    super.events.listenTo(eventName: "onAdFlowContainerHeightChanged", action: onAdFlowContainerHeightChanged)
    super.events.listenTo(eventName: "onAdFlowContainerShow", action: onAdFlowContainerShow)
    super.events.listenTo(eventName: "onValidationFailed", action: onValidationFailed)
    super.events.listenTo(eventName: "onError", action: onError)
    super.events.listenTo(eventName: "onNotified", action: onNotified)
  }

  required init?(coder aDecoder: NSCoder) {
    fatalError("init(coder:) has not been implemented")
  }

  override func didSetProps(_ changedProps: [String]!) {
    print("changeProps: \(changedProps)")
    super.updateView(changedProps)
  }

  func onAdFlowContainerClosed(data: Any?) {
    print(">> onAdFlowContainerClosed -> \n\(data)")
    AdsViewEventEmitter.emitter.sendEvent(withName: "onAdFlowContainerClosed", body: [data])
  }

  func onValidationFailed(data: Any?) {
    print(">> onValidationFailed -> \n\(data)")
    AdsViewEventEmitter.emitter.sendEvent(withName: "onValidationFailed", body: [data])
  }

  func onError(data: Any?) {
    print(">> onError -> \n\(data)")
    AdsViewEventEmitter.emitter.sendEvent(withName: "onError", body: [data])
  }

  func onAdFlowContainerShow(data: Any?) {
    print(">> onAdFlowContainerShow -> \n\(data)")
    AdsViewEventEmitter.emitter.sendEvent(withName: "onAdFlowContainerShow", body: [data])
  }

  func onAdFlowContainerHeightChanged(data: Any?) {
    print(">> onAdFlowContainerHeightChanged -> \n\(data)")    
    AdsViewEventEmitter.emitter.sendEvent(withName: "onAdFlowContainerHeightChanged", body: [data])
  }

  func onNotified(data: Any?) {
    print(">> onNotified -> \n\(data)")    
    AdsViewEventEmitter.emitter.sendEvent(withName: "onNotified", body: [data])
  }
}

@objc(AdsViewEventEmitter)
open class AdsViewEventEmitter: RCTEventEmitter {

  public static var emitter: AdsViewEventEmitter!

  override init() {
    super.init()
    AdsViewEventEmitter.emitter = self
  }

  @objc
  override public func supportedEvents() -> [String] {
    ["onReady", "onPending", "onError", "onValidationFailed", "onAdFlowContainerClosed", "onAdFlowContainerShow", "onAdFlowContainerHeightChanged", "onNotified"] 
  }

  @objc override public static func requiresMainQueueSetup() -> Bool {
        return false
    }
}
