/**
 * fluent-adflow-widget.js
 *
 * Licensed under the Fluent Software Development Kit (SDK) Terms of Use
 * Version 1.0 (the "License");
 *
 * You may not use this file except in compliance with the License.
 *
 * You may obtain a copy of the License at https://fluentco.com
 */

import {
  requireNativeComponent,
  NativeEventEmitter,
  NativeModules,
  type HostComponent,
} from 'react-native';
import React, { Component } from 'react';

const eventManagerEmitter = new NativeEventEmitter(NativeModules.AdsViewEventEmitter);

type FluentAdFlowWidgetProps = {
  style?: any;
  params: any;
  onAdShow?: any;
};
export interface  FluentAdFlowWidgetState {
  height: number;
}

export class FluentAdFlowWidgetView extends Component<FluentAdFlowWidgetProps, FluentAdFlowWidgetState> {
  subscriberOnAdFlowContainerClosed = eventManagerEmitter.addListener("onAdFlowContainerClosed", result => {
    this.props.onAdShow && this.props.onAdShow(false);
    console.log(">> onAdFlowContainerClosed -> ", result);
    this.setState({height: 0})
  });

  subscriberOnAdFlowContainerShow = eventManagerEmitter.addListener("onAdFlowContainerShow", result => {
    console.log(">> onAdFlowContainerShow -> ", result);
    this.props.onAdShow && this.props.onAdShow(true);
    // this.setState({height: this.props.style.height})
  });

  subscriberOnAdFlowContainerHeightChanged = eventManagerEmitter.addListener("onAdFlowContainerHeightChanged", result => {
    console.log(">> onAdFlowContainerHeightChanged -> ", result);
    this.setState({ height: parseInt(result) })
  });

  subscriberOnError = eventManagerEmitter.addListener("onError", result => {
    console.log(">> onError -> ", result);
    this.setState({height: 0})
  });

  subscriberOnValidationFailed = eventManagerEmitter.addListener("onValidationFailed", result => {
    console.log(">> onError -> ", result);
  });

  subscriberOnNotified = eventManagerEmitter.addListener("onNotified", result => {
    console.log(">> onNotified -> ", result);
  });

  constructor(props: FluentAdFlowWidgetProps) {
    super(props);
    this.state = { height: 0 };
  }

  override render() {
    return (
      <WidgetNativeComponent
        style={[this.props.style, { height: this.state.height }]}
        params={this.props.params}
      />
    );
  }

  override componentWillUnmount() {
    this.subscriberOnAdFlowContainerClosed.remove();
    this.subscriberOnAdFlowContainerShow.remove();
    this.subscriberOnAdFlowContainerHeightChanged.remove()
    this.subscriberOnError.remove();
    this.subscriberOnValidationFailed.remove()
    this.subscriberOnNotified.remove()
  }

}

export const WidgetNativeComponent: HostComponent<FluentAdFlowWidgetProps> =
  requireNativeComponent('FluentAdFlowWidgetView')


export default FluentAdFlowWidgetView;
